#!/usr/bin/env python
#
# test_webpage.py -
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#


import os
import os.path as op

import unittest.mock as mock

import fsleyes_widgets.utils.webpage as webpage


def test_fileToUrl():

    # fname, expected
    testcases = [
        '/home/blah/file.html'
        'file.html',
        './file.html',
        '../file.html',
    ]

    for fname in testcases:
        expect = op.abspath(fname)
        expect = f'file://{expect}'
        assert webpage.fileToUrl(fname) == expect


def test_openPage():

    opened = [None]

    def mockOpen(page):
        opened[0] = page

    tomock = 'fsleyes_widgets.utils.webpage.webbrowser.open'
    toopen = 'fakeUrl'
    with mock.patch(tomock, side_effect=mockOpen):

        webpage.openPage(toopen)

    assert opened[0] == toopen


def test_openFile():

    opened = [None]

    def mockOpen(page):
        opened[0] = page

    tomock   = 'fsleyes_widgets.utils.webpage.webbrowser.open'
    toopen   = '/path/to/file.html'
    expected = webpage.fileToUrl(toopen)

    with mock.patch(tomock, side_effect=mockOpen):
        webpage.openFile(toopen)

    assert opened[0] == expected



#!/usr/bin/env python
#
# test_elistbox.py -
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#


import random

import wx

from . import run_with_wx, simclick, simtext, simkey, realYield

from fsleyes_widgets import isalive
import fsleyes_widgets.elistbox as elistbox


def test_create():
    run_with_wx(_test_create)
def _test_create():
    frame = wx.GetApp().GetTopWindow()

    items    = random.sample(range(100), 5)
    labels   = [str(i)                 for i in items]
    tooltips = ['tip: {}'.format(i)    for i in items]
    listbox  = elistbox.EditableListBox(
        frame,
        labels=labels,
        clientData=items,
        tooltips=tooltips)

    assert listbox.VisibleItemCount() == 5
    assert listbox.GetCount()         == 5

    assert listbox.GetSelection() == wx.NOT_FOUND
    listbox.SetSelection(3)
    assert listbox.GetSelection() == 3

    for idx, i in enumerate(items):
        assert listbox.IndexOf(i) == idx

    assert list(listbox.GetLabels()) == list(labels)
    assert list(listbox.GetData())   == list(items)
    for i in range(len(items)):
        assert listbox.GetItemTooltip(i) == tooltips[i]
        assert listbox.GetItemLabel(  i) == labels[i]
        assert listbox.GetItemData(   i) == items[i]


def test_add_remove():
    run_with_wx(_test_add_remove)
def _test_add_remove():
    frame   = wx.GetApp().GetTopWindow()
    items   = [str(random.randint(1, 100)) for i in range(5)]
    listbox = elistbox.EditableListBox(frame, items)

    # delete
    for i in range(5):
        ridx = random.randint(0, len(items) - 1)
        items.pop(ridx)
        listbox.Delete(ridx)
        assert list(listbox.GetLabels()) == items

    # append
    for i in range(5):
        item = str(random.randint(0, 100))
        items.append(item)
        listbox.Append(item)
        assert list(listbox.GetLabels()) == items

    # insert
    for i in range(5):
        item = str(random.randint(0, 100))
        idx  = random.randint(0, len(items) - 1)
        items.insert(idx, item)
        listbox.Insert(item, idx)
        assert list(listbox.GetLabels()) == items


def test_hidden():
    run_with_wx(_test_hidden)
def _test_hidden():

    class FakeEv(object):
        def __init__(self, key):
            self.key = key
        def GetKeyCode(self):
            return self.key
        def HasModifiers(self):
            return False
        def Skip(self):
            pass

    frame   = wx.GetApp().GetTopWindow()
    items   = list(range(10))
    labels  = [str(i) for i in items]
    listbox = elistbox.EditableListBox(
        frame,
        labels=labels,
        clientData=items)

    def hide(label, data):
        return data % 2 == 0

    listbox.SetSelection(0)
    listbox.MoveItem(5)
    assert listbox.GetSelection() == 5
    assert listbox.GetItemData() == [1, 2, 3, 4, 5, 0, 6, 7, 8, 9]

    listbox.MoveItem(-5)
    assert listbox.GetSelection() == 0
    assert listbox.GetItemData() == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    listbox.ApplyFilter(hide)
    assert listbox.GetVisibleItemData() == [0, 2, 4, 6, 8]
    assert listbox.GetSelection() == 0

    listbox._onKeyboard(FakeEv(wx.WXK_DOWN))
    assert listbox.GetSelection() == 2
    listbox._onKeyboard(FakeEv(wx.WXK_UP))
    assert listbox.GetSelection() == 0

    listbox.MoveItem(1)
    assert listbox.GetSelection() == 2
    assert listbox.GetVisibleItemData() == [2, 0, 4, 6, 8]
    assert listbox.GetItemData() == [1, 2, 0, 3, 4, 5, 6, 7, 8, 9]



#!/usr/bin/env python
#
# test_colourbarbitmap.py -
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#


import os.path   as op
import itertools as it

import numpy as np

import matplotlib        as mpl
import matplotlib.colors as colors
import matplotlib.image  as mplimg

import pytest

import fsleyes_widgets.utils.colourbarbitmap as cbarbmp

from . import compare_images


datadir = op.join(op.dirname(__file__), 'testdata', 'colourbarbitmap')


def gen_file_id(*args):
    """Generate a "safe" string for use in file names. """
    fid = '_'.join(map(str, args))
    for c in '[](){}, !@#$%&*;:=<>|/?\\"\'':
        fid = fid.replace(c, '_')

    while '__' in fid:         fid = fid.replace('__', '_')
    while fid.endswith('_'):   fid = fid[:-1]
    while fid.startswith('_'): fid = fid[1:]
    return fid


def _compare(bmp, fname):

    bmp       = bmp.transpose((1, 0, 2))
    fname     = op.join(datadir, fname)


    if op.exists(fname):
        benchmark = mplimg.imread(fname) * 255
        result = compare_images(bmp, benchmark, 0.05)
    else:
        result = [False]

    if not result[0]:
        basedir = op.join(op.dirname(__file__), '..')
        mplimg.imsave(op.join(basedir, op.basename(fname)), bmp)

    return result[0]


def test_standard_usage():

    cmaps           = ['Greys', 'Blues']
    cmapResolutions = [6, 256]
    orientations    = ['vertical', 'horizontal']
    alphas          = [0.25, 1.0]
    bgColours       = [(0, 0, 0, 0), (0, 0, 0, 1), (1, 0, 0, 1)]
    size            = (100, 25)

    testcases = it.product(cmaps,
                           cmapResolutions,
                           orientations,
                           alphas,
                           bgColours)

    for testcase in testcases:
        cmap, cmapRes, orient, alpha, bgColour = testcase

        if orient == 'vertical': height, width = size
        else:                    width, height = size

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      cmapResolution=cmapRes,
                                      alpha=alpha,
                                      orientation=orient,
                                      bgColour=bgColour)

        fname = gen_file_id('standard_usage', *testcase)
        fname = f'{fname}.png'

        assert _compare(bmp, fname)


def test_negCmap_invert():

    # cmap, negCmap, invert, orientation
    testcases = [
        ('Reds',  None,   False, 'vertical'),
        ('Reds',  None,   False, 'horizontal'),
        ('Reds',  None,   True,  'vertical'),
        ('Reds',  None,   True,  'horizontal'),
        ('Reds', 'Blues', False, 'vertical'),
        ('Reds', 'Blues', False, 'horizontal'),
        ('Reds', 'Blues', True,  'vertical'),
        ('Reds', 'Blues', True,  'horizontal')
    ]

    size = 100, 25

    for cmap, negCmap, invert, orient in testcases:

        fname = gen_file_id('negCmap_invert', cmap, negCmap, invert, orient)
        fname = f'{fname}.png'

        if orient == 'vertical': height, width = size
        else:                    width, height = size

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      negCmap=negCmap,
                                      invert=invert,
                                      orientation=orient)

        assert _compare(bmp, fname)


def test_gamma():

    gammas        = [0.5, 1.0, 2.0, 3.0]
    inverts       = [False, True]
    negCmaps      = [False, True]
    cmaps         = [('Greys', 'Greys'), ('Reds', 'Blues')]
    width, height = 100, 25

    testcases = it.product(gammas, inverts, negCmaps, cmaps)

    for gamma, invert, useNegCmap, cmap in testcases:

        cmap, negCmap = cmap

        if not useNegCmap:
            negCmap = None

        fname = gen_file_id('gamma', gamma, invert, cmap, negCmap)
        fname = f'{fname}.png'

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      invert=invert,
                                      orientation='horizontal',
                                      gamma=gamma,
                                      negCmap=negCmap)

        assert _compare(bmp, fname)


def test_logScaleRange_interp():

    # register a low-res colour map to test interpolation
    rgbs = np.array([[1, 0, 0],
                     [1, 1, 1],
                     [0, 1, 0]])
    cmap = colors.ListedColormap(rgbs, name='MYCMAP')
    mpl.colormaps.register(cmap, name='MYCMAP')


    logScales     = [None, (0, 1), (50, 100)]
    interps       = [False, True]
    reses         = [10, 100]
    cmaps         = ['hot', 'MYCMAP']
    width, height = 100, 25

    testcases = it.product(logScales, interps, reses, cmaps)

    for logScale, interp, res, cmap in testcases:

        fname = gen_file_id('logScaleRange_interp',
                            logScale, interp, res, cmap)
        fname = f'{fname}.png'

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      cmapResolution=res,
                                      interp=interp,
                                      logScaleRange=logScale,
                                      orientation='horizontal')

        assert _compare(bmp, fname)


def test_label():

    orientations = ['horizontal', 'vertical']
    labelsides   = ['top', 'bottom', 'left', 'right']
    fontsizes    = [6, 10, 16]
    textcolours  = [(0, 0, 0, 1), (1, 0, 0, 1)]

    testcases = it.product(orientations, labelsides, fontsizes, textcolours)

    for orient, side, size, colour in testcases:

        if orient[0] == 'v': width, height = 100, 300
        else:                width, height = 300, 100

        bmp = cbarbmp.colourBarBitmap('Reds',
                                      width,
                                      height,
                                      label='Label',
                                      orientation=orient,
                                      labelside=side,
                                      fontsize=size,
                                      textColour=colour)

        fname = gen_file_id('label', orient, side, size, colour)
        fname = f'{fname}.png'

        assert _compare(bmp, fname)


def test_ticks():
    orientations = ['horizontal', 'vertical']
    labelsides   = ['top', 'bottom', 'left', 'right']
    fontsizes    = [6, 10, 16]
    textcolours  = [(0, 0, 0, 1), (1, 0, 0, 1)]

    tickss      = [[0.0, 0.25, 0.75, 1.0],
                   [0.25, 0.75]]

    testcases = it.product(orientations,
                           labelsides,
                           fontsizes,
                           textcolours,
                           tickss)

    for orient, side, size, colour, ticks in testcases:

        labels = [str(t) for t in ticks]

        if orient[0] == 'v': width, height = 100, 300
        else:                width, height = 300, 100

        bmp = cbarbmp.colourBarBitmap('Reds',
                                      width,
                                      height,
                                      ticks=ticks,
                                      ticklabels=labels,
                                      orientation=orient,
                                      labelside=side,
                                      fontsize=size,
                                      textColour=colour)

        fname = gen_file_id('ticks', orient, side, size, colour, labels)
        fname = f'{fname}.png'

        assert _compare(bmp, fname)


def test_tickalign():

    orientations = ['horizontal', 'vertical']
    labelsides   = ['top', 'bottom']

    ticks      = [0.0, 0.25, 0.5, 0.75, 1.0]
    ticklabels = [str(t) for t in ticks]
    tickaligns = [
        ['left'   for t in ticks],
        ['right'  for t in ticks],
        ['center' for t in ticks],
        ['left', 'center', 'center', 'center', 'right']]

    testcases = it.product(orientations, labelsides, tickaligns)

    for orient, side, align in testcases:

        if orient[0] == 'v': width, height = 100, 300
        else:                width, height = 300, 100

        bmp = cbarbmp.colourBarBitmap('Blues',
                                      width,
                                      height,
                                      ticks=ticks,
                                      ticklabels=ticklabels,
                                      tickalign=align,
                                      orientation=orient,
                                      labelside=side,
                                      textColour=(0, 0, 0, 1))


        falign = ''.join([a[0] for a in align])
        fname  = gen_file_id('tickalign', orient, side, falign)
        fname  = f'{fname}.png'

        assert _compare(bmp, fname)


def test_label_and_ticks():

    orientations = ['horizontal', 'vertical']
    labelsides   = ['top', 'bottom']
    fontsizes    = [6, 10, 16]

    testcases = it.product(orientations,
                           labelsides,
                           fontsizes)

    ticks      = [0.2, 0.5, 0.8]
    ticklabels = [str(t) for t in ticks]
    label      = 'Label'

    for orient, side, size, in testcases:

        if orient[0] == 'v': width, height = 100, 300
        else:                width, height = 300, 100

        bmp = cbarbmp.colourBarBitmap('Reds',
                                      width,
                                      height,
                                      ticks=ticks,
                                      ticklabels=ticklabels,
                                      label=label,
                                      orientation=orient,
                                      labelside=side,
                                      fontsize=size,
                                      textColour=(0, 0, 0.2, 1))

        fname = gen_file_id('label_and_ticks', orient, side, size)
        fname = f'{fname}.png'

        assert _compare(bmp, fname)


def test_scale():
    scales       = [0.5, 1.0, 2.0, 3.0]
    orientations = ['horizontal', 'vertical']
    labelsides   = ['top', 'bottom']

    testcases = it.product(scales,
                           orientations,
                           labelsides)

    ticks      = [0.1, 0.9]
    ticklabels = [str(t) for t in ticks]
    label      = 'Label'

    for scale, orient, side in testcases:

        if orient[0] == 'v': width, height = 100, 300
        else:                width, height = 300, 100

        bmp = cbarbmp.colourBarBitmap('Reds',
                                      width,
                                      height,
                                      ticks=ticks,
                                      ticklabels=ticklabels,
                                      label=label,
                                      orientation=orient,
                                      labelside=side,
                                      scale=scale,
                                      textColour=(0, 0, 0.2, 1))

        fname = gen_file_id('scale', scale, orient, side)
        fname = f'{fname}.png'
        assert _compare(bmp, fname)


def test_negCmap_ticks():
    testcases = [
        ('Reds', 'Blues', [0, 1]),
        ('Reds', 'Blues', [0, 0.5, 1]),
        ('Reds', 'Blues', [0, 0.49, 0.51, 1]),
        ('Reds', 'Blues', [0, 0.25, 0.5, 0.75, 1])
    ]

    for cmap, negCmap, ticks in testcases:

        fname = gen_file_id(cmap, negCmap, ticks)
        fname = f'negCmap_invert_ticks_{fname}.png'

        ticklabels = ['{:0.2f}'.format(t) for t in ticks]
        tickalign = ['left'] + ['center'] * (len(ticks) - 2) + ['right']

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      800,
                                      100,
                                      fontsize=30,
                                      negCmap=negCmap,
                                      orientation='horizontal',
                                      ticks=ticks,
                                      tickalign=tickalign,
                                      ticklabels=ticklabels)
        assert _compare(bmp, fname)


def test_modAlpha():

    cmaps         = ['Reds']
    alphas        = [0.5, 1.0]
    bgColours     = [(0, 0, 0, 0), (0, 0, 0, 1), (1, 0, 0, 1)]
    dispRanges    = [(-5, 5)]
    modRanges     = [(-2.5, 2.5), (-5, 5), (-10, -5), (5, 10)]
    invModAlphas  = [False, True]
    width, height = 300, 100


    testcases = it.product(cmaps,
                           alphas,
                           bgColours,
                           dispRanges,
                           modRanges,
                           invModAlphas)

    for testcase in testcases:
        cmap, alpha, bgColour, dispRange, modRange, invModAlpha = testcase

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      alpha=alpha,
                                      bgColour=bgColour,
                                      orientation='horizontal',
                                      modAlpha=True,
                                      invModAlpha=invModAlpha,
                                      displayRange=dispRange,
                                      modRange=modRange)

        fname = gen_file_id('modAlpha', *testcase)
        fname = f'{fname}.png'

        assert _compare(bmp, fname)


def test_modAlpha_negCmap():

    cmaps         = ['Reds']
    negCmaps      = ['Blues']
    alphas        = [0.5, 1.0]
    bgColours     = [(0, 0, 0, 0), (0, 0, 0, 1), (1, 0, 0, 1)]
    dispRanges    = [(0, 5)]
    modRanges     = [(1.25, 3.75), (0, 5), (5, 10)]
    invModAlphas  = [False, True]
    width, height = 300, 100

    testcases = it.product(cmaps,
                           negCmaps,
                           alphas,
                           bgColours,
                           dispRanges,
                           modRanges,
                           invModAlphas)

    for testcase in testcases:
        cmap, negCmap, alpha, bgColour, dispRange, modRange, invModAlpha = testcase

        bmp = cbarbmp.colourBarBitmap(cmap,
                                      width,
                                      height,
                                      negCmap=negCmap,
                                      alpha=alpha,
                                      bgColour=bgColour,
                                      orientation='horizontal',
                                      modAlpha=True,
                                      invModAlpha=invModAlpha,
                                      displayRange=dispRange,
                                      modRange=modRange)

        fname = gen_file_id('modAlpha', *testcase)
        fname = f'{fname}.png'
        assert _compare(bmp, fname)


def test_badargs():

    testcases = [
        dict(cmap='badcmap'),
        dict(orientation='badorient'),
        dict(labelside='badside'),
        dict(ticks=[0, 0.5, 1],
             ticklabels=['l', 'l', 'l'],
             tickalign=['badalign', 'badalign', 'badalign']),
    ]

    # bad colour map
    for t in testcases:
        with pytest.raises(Exception):
            cbarbmp.colourBarBitmap(t.get('cmap', 'Reds'), 50, 50, **t)



#!/usr/bin/env python
#
# colourbarbitmap.py - A function which renders a colour bar using
# matplotlib as an RGBA bitmap.
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#
"""This module provides a single function, :func:`colourBarBitmap`, which uses
:mod:`matplotlib` to plot a colour bar. The colour bar is rendered off-screen
and returned as an RGBA bitmap.
"""


def colourBarBitmap(cmap,
                    width,
                    height,
                    cmapResolution=256,
                    negCmap=None,
                    invert=False,
                    gamma=1,
                    ticks=None,
                    ticklabels=None,
                    tickalign=None,
                    label=None,
                    orientation='vertical',
                    labelside='top',
                    alpha=1.0,
                    fontsize=10,
                    bgColour=None,
                    textColour='#ffffff',
                    scale=1.0,
                    interp=False,
                    logScaleRange=None,
                    modAlpha=False,
                    invModAlpha=False,
                    displayRange=None,
                    modRange=None):
    """Plots a colour bar using :mod:`matplotlib`.


    The rendered colour bar is returned as a RGBA bitmap within a
    ``numpy.uint8`` array of size :math:`w \\times h \\times 4`, with the
    top-left pixel located at index ``[0, 0, :]``.


    A rendered colour bar will look something like this:

    .. image:: images/colourbarbitmap.png
       :scale: 50%
       :align: center


    :arg cmap:           Name of a registered :mod:`matplotlib` colour map.

    :arg width:          Colour bar width in pixels.

    :arg height:         Colour bar height in pixels.

    :arg cmapResolution: Colour map resolution (number of distinct colours).

    :arg negCmap:        If provided, two colour maps are drawn, centered at 0.

    :arg invert:         If ``True``, the colour map is inverted.

    :arg gamma:          Gamma correction factor - exponentially weights the
                         colour map scale towards one end.

    :arg ticks:          Locations of tick labels. Ignored if
                         ``ticklabels is None``.

    :arg ticklabels:     Tick labels.

    :arg tickalign:      Tick alignment (one for each tick, either ``'left'``,
                         ``'right'``, or ``'center'``).

    :arg label:          Text label to show next to the colour bar.

    :arg orientation:    Either ``vertical`` or ``horizontal``.

    :arg labelside:      Side of the colour bar to put the label - ``top``,
                         ``bottom``, ``left`` or ``right``. If
                         ``orientation='vertical'``, then ``top``/``bottom``
                         are interpreted as ``left``/``right`` (and vice-versa
                         when ``orientation='horizontal'``).

    :arg alpha:          Colour bar transparency, in the range ``[0.0 - 1.0]``.

    :arg fontsize:       Label font size in points.

    :arg bgColour:       Background colour - can be any colour specification
                         that is accepted by :mod:`matplotlib`.

    :arg textColour:     Label colour - can be any colour specification that
                         is accepted by :mod:`matplotlib`.

    :arg scale:          DPI scaling factor.

    :arg interp:         If true, and the colour map has fewer colours than
                         ``cmapResolution``, it is linearly interpolated
                         to have ``cmapResolution``.

    :arg logScaleRange:  Tuple containing ``(min, max)`` display range to which
                         the colour bar is mapped to. If provided, the colour
                         bar will be scaled to the natural logarithm of the
                         display range.

    :arg modAlpha:       If ``True``, modulate the colour bar transparency
                         according to the modulate and display ranges, so that
                         colours at or below the low modulation range are
                         fully transparent, and colours at or above the high
                         modulation range have transparency set to ``alpha``.

    :arg invModAlpha:    If ``True``, flips the direction in which the colour
                         bar is modulated by transparency.

    :arg displayRange:   Low/high values corresponding to the low/high colours.
                         Required when ``modAlpha=True``.

    :arg modRange:       Low/high values between which transparency should be
                         modulated. Required when ``modAlpha=True``.
    """

    # These imports are expensive, so we're
    # importing at the function level.
    import numpy                           as np
    import matplotlib.backends.backend_agg as mplagg
    import matplotlib.figure               as mplfig

    if orientation not in ['vertical', 'horizontal']:
        raise ValueError('orientation must be vertical or '
                         f'horizontal ({orientation})')

    if orientation == 'horizontal':
        if   labelside == 'left':  labelside = 'top'
        elif labelside == 'right': labelside = 'bottom'
    else:
        if   labelside == 'top':    labelside = 'left'
        elif labelside == 'bottom': labelside = 'right'

    if labelside not in ['top', 'bottom', 'left', 'right']:
        raise ValueError('labelside must be top, bottom, '
                         f'left or right ({labelside})')

    if modAlpha:
        try:
            _, _ = displayRange
            _, _ = modRange
        except Exception:
            raise ValueError('displayRange and modRange must be '
                             'provided when modAlpha is True')

    # vertical plots are rendered horizontally,
    # and then simply rotated at the end
    if orientation == 'vertical':
        width, height = height, width
        if labelside == 'left': labelside = 'top'
        else:                   labelside = 'bottom'

    # Default is 96 dpi to an inch
    winches   = width  / 96.0
    hinches   = height / 96.0
    dpi       = scale  * 96.0
    ncols     = cmapResolution
    data      = genColours(cmap, ncols, invert, alpha,
                           gamma, interp, logScaleRange,
                           modAlpha, invModAlpha,
                           displayRange, modRange)

    if negCmap is not None:
        ndata  = genColours(negCmap, ncols, not invert, alpha,
                            gamma, interp, logScaleRange,
                            modAlpha, invModAlpha,
                            displayRange, modRange)
        data   = np.concatenate((ndata, data), axis=0)
        ncols *= 2

    # Turn from a 1D rgba vector into a 2D RGBA image
    data = np.dstack((data, data)).transpose((2, 0, 1))

    # force tick positions to
    # the left edge of the
    # corresponding colour
    if ticks is not None:
        ticks = [t - 0.5 / ncols for t in ticks]

    fig     = mplfig.Figure(figsize=(winches, hinches), dpi=dpi)
    canvas  = mplagg.FigureCanvasAgg(fig)
    ax      = fig.add_subplot(111)

    if bgColour is not None:
        fig.patch.set_facecolor(bgColour)
        ax.set_facecolor(bgColour)
    else:
        fig.patch.set_alpha(0)
        ax.set_alpha(0)

    # draw the colour bar
    ax.imshow(data,
              aspect='auto',
              origin='lower',
              interpolation='nearest')

    ax.set_yticks([])
    ax.tick_params(colors=textColour, labelsize=fontsize, length=0)

    if labelside == 'top':
        ax.xaxis.tick_top()
        ax.xaxis.set_label_position('top')
    else:
        ax.xaxis.tick_bottom()
        ax.xaxis.set_label_position('bottom')

    if label is not None:
        ax.set_xlabel(label, fontsize=fontsize, color=textColour)
        label = ax.xaxis.get_label()

    if ticks is None or ticklabels is None:
        ax.set_xticks([])
    else:

        ax.set_xticks(np.array(ticks) * ncols)
        ax.set_xticklabels(ticklabels)
        ticklabels = ax.xaxis.get_ticklabels()

    # Resize the colour bar to make
    # space for the ticks and labels
    wpad        = 5 / float(width)
    hpad        = 5 / float(height)
    totalHeight = 1.0

    # Figure out text height in
    # pixels (with 6 pixel padding)
    fontpix    = (fontsize / 72.0) * dpi / scale
    textHeight = (fontpix + 6) / float(height)

    if label      is not None: totalHeight -= textHeight
    if ticklabels is not None: totalHeight -= textHeight

    if labelside == 'top': bottom, top = 0, totalHeight
    else:                  bottom, top = 1 - totalHeight, 1

    fig.subplots_adjust(
        left=wpad,
        right=1.0 - wpad,
        bottom=bottom + hpad,
        top=top - hpad)

    # Divide the vertical height between the
    # colour bar, the label, and the tick labels.
    if label is not None:

        # I don't understand why, but I have
        # to set va to the opposite of what
        # I would have thought.
        if labelside == 'top':
            label.set_va('bottom')
            label.set_position((0.5, 1.0))
        else:
            label.set_va('top')
            label.set_position((0.5, 0))

    # Adjust tick label horizontal alignment. This
    # must be done *after* calling tick_top/tick_bottom,
    # as I think the label objects get recreated.
    if ticklabels is not None and tickalign is not None:
        for l, a in zip(ticklabels, tickalign):
            l.set_horizontalalignment(a)

    ax.set_xlim((-0.5, ncols - 0.5))

    canvas.draw()

    buf = canvas.tostring_argb()
    ncols, nrows = canvas.get_width_height()

    bitmap = np.frombuffer(buf, dtype=np.uint8)
    bitmap = bitmap.reshape(nrows, ncols, 4).transpose([1, 0, 2])

    # the bitmap is in argb order,
    # but we want it in rgba
    rgb = bitmap[:, :, 1:]
    a   = bitmap[:, :, 0]
    bitmap = np.dstack((rgb, a))

    if orientation == 'vertical':
        bitmap = np.flipud(bitmap.transpose([1, 0, 2]))
        bitmap = np.rot90(bitmap, 2)

    return bitmap


def genColours(cmap,
               ncols,
               invert,
               alpha,
               gamma=1,
               interp=False,
               logScaleRange=None,
               modAlpha=False,
               invModAlpha=False,
               displayRange=None,
               modRange=None):
    """Generate an array containing ``ncols`` colours from the given
    colour map object/function.
    """

    import numpy      as np
    import matplotlib as mpl

    # cmap can be a mpl colormap object or name
    if isinstance(cmap, str):
        cmap  = mpl.colormaps[cmap]

    # Map display range to colour map logarithmically
    if logScaleRange is not None:
        dmin, dmax    = logScaleRange
        idxs          = np.linspace(dmin, dmax, ncols)
        idxs          = np.log(idxs)
        finite        = np.isfinite(idxs)
        imax          = idxs[finite].max()
        imin          = idxs[finite].min()
        idxs          = (idxs - imin) / (imax - imin)
        idxs[~finite] = 0

    # Map display range to colour map linearly
    else:
        idxs = np.linspace(0.0, 1.0, ncols)

    if gamma not in (None, 1):
        idxs = idxs ** gamma

    if invert:
        idxs = idxs[::-1]

    # interpolate if requested, and if the
    # number of colours in the cmap does
    # not match the requested resolution
    if interp and (ncols != cmap.N):
        rawidxs = np.linspace(0, 1, cmap.N)
        rawrgbs = cmap(rawidxs)
        rgbs    = np.zeros((ncols, 4), dtype=np.float32)
        for chan in range(3):
            rgbs[:, chan] = np.interp(idxs, rawidxs, rawrgbs[:, chan])
    else:
        rgbs = cmap(idxs)

    if not modAlpha:
        rgbs[:, 3] = alpha

    else:
        dmin, dmax = displayRange
        mmin, mmax = modRange
        amin, amax = 0, alpha

        if invModAlpha:
            amin, amax = amax, amin

        # mod range normalised to the range [0, 1]
        nmmin = (mmin - dmin) / (dmax - dmin)
        nmmax = (mmax - dmin) / (dmax - dmin)
        nmmin = np.clip(nmmin, 0, 1)
        nmmax = np.clip(nmmax, 0, 1)

        # mod range in terms of num colours (ncols)
        immin = int(np.round(ncols * nmmin))
        immax = int(np.round(ncols * nmmax))

        # mod range below or above display range
        if   nmmin == 1: rgbs[:, 3] = amin
        elif nmmax == 0: rgbs[:, 3] = amax

        # mod range overlapping with display range
        else:
            alphas              = np.zeros(ncols, dtype=np.float32)
            alphas[:immin]      = amin
            alphas[immax:]      = amax
            alphas[immin:immax] = np.linspace(amin, amax, abs(immax - immin))

            if invert: rgbs[:, 3] = alphas[::-1]
            else:      rgbs[:, 3] = alphas

    return rgbs
